@@ -20,6 +20,7 @@ import logging
 
 import numpy as np
 import scipy.io as spio
+import cProfile
 
 from hdidx.distance import distFunc
 from hdidx import indexer
@@ -50,7 +51,7 @@ def load_random(ntrain, nbase, nquery, d=16):
     return vtrain, vbase, vquery, ids_gnd
 
 
-def create_random_data(ntrain=10**4, nbase=10**4, nquery=10**1):
+def create_random_data(ntrain=10**4, nbase=10**4, nquery=10**2):
     """
     Create random data
     """
@@ -82,7 +83,7 @@ def compute_stats(nquery, ids_gnd, ids_pqc, k):
         if i > k:
             break
         r_at_i = (nn_ranks_pqc < i).sum() * 100.0 / nquery
-        logging.warning('r@%3d = %.3f' % (i, r_at_i))
+        logging.warning('\rr@%3d = %.3f' % (i, r_at_i))
 
 
 class TestPQNew(unittest.TestCase):
@@ -98,19 +99,61 @@ class TestPQNew(unittest.TestCase):
             create_random_data()
         cls.nsubq = 8
         cls.topk = 100
+        cls.coarsek = 32
 
     @classmethod
     def tearDownClass(cls):
         pass
 
-    def test_pq_mem(self):
-        idx = indexer.PQIndexer()
+    def test_ivfpq_lmdb_0_build_save_add_search(self):
+        idx = indexer.IVFPQIndexer()
         idx.build({
             'vals': self.vtrain,
             'nsubq': self.nsubq,
+            'coarsek': self.coarsek,
         })
-        idx.save('mem.info')
-        idx.set_storage('mem')
+        idx.save('ivf_lmdb.info')
+        idx.set_storage('lmdb', {
+            'path': 'ivf_lmdb.idx',
+            'clear': True,
+        })
+        idx.add(self.vbase)
+        ids, dis = idx.search(self.vquery, topk=self.topk)
+        # cProfile.runctx('ids, dis = idx.search(self.vquery, topk=self.topk)',
+        #                 None, locals())
+        compute_stats(self.vquery.shape[0], self.ids_gnd, ids, self.topk)
+
+    def test_ivfpq_lmdb_1_SKIP_load_add_search(self):
+        idx1 = indexer.IVFPQIndexer()
+        idx1.load('ivf_lmdb.info')
+        idx1.set_storage('lmdb', {
+            'path': 'ivf_lmdb.idx',
+            'clear': True,
+        })
+        idx1.add(self.vbase)
+        ids, dis = idx1.search(self.vquery, topk=self.topk)
+        compute_stats(self.vquery.shape[0], self.ids_gnd, ids, self.topk)
+
+    def test_ivfpq_lmdb_2_SKIP_load_SKIP_search(self):
+        idx2 = indexer.IVFPQIndexer()
+        idx2.load('ivf_lmdb.info')
+        idx2.set_storage('lmdb', {
+            'path': 'ivf_lmdb.idx',
+            'clear': False,
+        })
+        ids, dis = idx2.search(self.vquery, topk=self.topk)
+        compute_stats(self.vquery.shape[0], self.ids_gnd, ids, self.topk)
+
+    def test_ivfpq_mem(self):
+        idx = indexer.IVFPQIndexer()
+
+        idx.build({
+            'vals': self.vtrain,
+            'nsubq': self.nsubq,
+            'coarsek': self.coarsek,
+        })
+        idx.save('ivf_mem.info')
+        idx.set_storage('mem', {})
 
         idx.add(self.vbase)
         ids, dis = idx.search(self.vquery, topk=self.topk)
@@ -152,6 +195,24 @@ class TestPQNew(unittest.TestCase):
         ids, dis = idx2.search(self.vquery, topk=self.topk)
         compute_stats(self.vquery.shape[0], self.ids_gnd, ids, self.topk)
 
+    def test_pq_mem(self):
+        idx = indexer.PQIndexer()
+        idx.build({
+            'vals': self.vtrain,
+            'nsubq': self.nsubq,
+        })
+        idx.save('mem.info')
+        idx.set_storage('mem')
+
+        idx.add(self.vbase)
+        ids, dis = idx.search(self.vquery, topk=self.topk)
+        compute_stats(self.vquery.shape[0], self.ids_gnd, ids, self.topk)
+
 
 if __name__ == '__main__':
-    unittest.main()
+    logging.warn("The results of mem storage and lmdb storage might be " +
+                 "different even if the database and queries are exactly " +
+                 "the same, this is because the randomization exists in " +
+                 "k-means clustering.")
+    unittest.main(failfast=True)
+    # cProfile.run('unittest.main(failfast=True)')
